# Copyright 2015 Digital Borderlands Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from cityhall import Settings, _validate_path
from cityhall.errors import InvalidCall
from unittest import TestCase
from helper_funcs import (
    build,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
)
from mock import patch
from datetime import datetime


class TestEnv(
    TestCase,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
):
    def setUp(self):
        self.url = 'http://not.a.real.url/api/'
        self.name = 'test_user'
        self.password = ''
        self.value = 'some value'
        self.protect = True
        self.get_value = build(
            update={'value': self.value, 'protect': self.protect}
        )
        self.get_children = build(
            update={
                "path": "/abc/",
                "children": [
                    {
                        "override": "",
                        "path": "/abc/val1/",
                        "id": 9,
                        "value": "1000",
                        "protect": False,
                        "name": "val1"
                    },
                    {
                        "override": "test_user",
                        "path": "/abc/val1/",
                        "id": 12,
                        "value": "50",
                        "protect": False,
                        "name": "val1"
                    }
                ]
            }
        )
        self.get_history = build(
            update={
                "History": [
                    {
                        "active": False,
                        "override": "",
                        "id": 12,
                        "value": "1000",
                        "datetime": datetime.now(),
                        "protect": False,
                        "name": "abc",
                        "author": "cityhall"
                    },
                    {
                        "active": True,
                        "override": "",
                        "id": 12,
                        "value": "50",
                        "datetime": datetime.now(),
                        "protect": False,
                        "name": "abc",
                        "author": "test_dev"
                    }
                ]
            }
        )
        with patch('requests.Session.post') as post:
            with patch('requests.Session.get') as get:
                post.return_value = build()
                get.return_value = build(update={'value': 'dev'})
                self.settings = Settings(self.url, self.name, self.password)

    def test_get_mixins(self):
        call = lambda: self.settings._get_raw('dev', '/abc', None)
        self.failed_call_honored(call, 'requests.Session.get')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.get')
    def test_get_no_override(self, get):
        get.return_value = self.get_value
        value = self.settings.get('/abc')
        get.assert_called_once_with(self.url + 'env/dev/abc/', params=None)
        self.assertEqual(self.value, value)

    @patch('requests.Session.get')
    def test_get_specify_protect(self, get):
        get.return_value = self.get_value
        ret = self.settings.get('/abc', view_raw=True)
        get.assert_called_once_with(self.url + 'env/dev/abc/', params=None)
        self.assertIsInstance(ret, dict)
        self.assertEqual(self.value, ret['value'])
        self.assertEqual(self.protect, ret['protect'])

    @patch('requests.Session.get')
    def test_get_specify_protect_and_override(self, get):
        get.return_value = self.get_value
        ret = self.settings.get('/abc', view_raw=True, override='guest')
        params = {'override': 'guest'}
        get.assert_called_once_with(self.url + 'env/dev/abc/', params=params)
        self.assertIsInstance(ret, dict)
        self.assertEqual(self.value, ret['value'])
        self.assertEqual(self.protect, ret['protect'])

    @patch('requests.Session.get')
    def test_get_with_override(self, get):
        get.return_value = self.get_value
        value = self.settings.get('/abc', override='guest')
        get_url = self.url + 'env/dev/abc/'
        get.assert_called_once_with(get_url, params={'override': 'guest'})
        self.assertEqual(self.value, value)

    @patch('requests.Session.get')
    def test_get_with_env(self, get):
        get.return_value = self.get_value
        value = self.settings.get('/abc', env='qa')
        get.assert_called_once_with(self.url + 'env/qa/abc/', params=None)
        self.assertEqual(self.value, value)

    @patch('requests.Session.get')
    def test_get_with_env_and_override(self, get):
        get.return_value = self.get_value
        value = self.settings.get('/abc', env='qa', override='guest')
        get_url = self.url + 'env/qa/abc/'
        get.assert_called_once_with(get_url, params={'override': 'guest'})
        self.assertEqual(self.value, value)

    @patch('requests.Session.get')
    def test_get_view_children(self, get):
        get.return_value = self.get_children
        children = self.settings.get_children('/abc')
        params = {'viewchildren': True}
        get_url = self.url + 'env/dev/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_children.json()
        self.assertEqual(children, json['children'])

    @patch('requests.Session.get')
    def test_get_view_children_override(self, get):
        get.return_value = self.get_children
        children = self.settings.get_children('/abc', override='guest')
        params = {'viewchildren': True, 'override': 'guest'}
        get_url = self.url + 'env/dev/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_children.json()
        self.assertEqual(children, json['children'])

    @patch('requests.Session.get')
    def test_get_view_children_override_env(self, get):
        get.return_value = self.get_children
        children = self.settings.get_children('/abc', override='abc', env='qa')
        params = {'viewchildren': True, 'override': 'abc'}
        get_url = self.url + 'env/qa/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_children.json()
        self.assertEqual(children, json['children'])

    @patch('requests.Session.get')
    def test_get_view_history(self, get):
        get.return_value = self.get_history
        history = self.settings.get_history('/abc')
        params = {'viewhistory': True}
        get_url = self.url + 'env/dev/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_history.json()
        self.assertEqual(history, json['History'])

    @patch('requests.Session.get')
    def test_get_view_history_override(self, get):
        get.return_value = self.get_history
        history = self.settings.get_history('/abc', override='abc')
        params = {'viewhistory': True, 'override': 'abc'}
        get_url = self.url + 'env/dev/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_history.json()
        self.assertEqual(history, json['History'])

    @patch('requests.Session.get')
    def test_get_view_history_override_env(self, get):
        get.return_value = self.get_history
        history = self.settings.get_history('/abc', override='abc', env='qa')
        params = {'viewhistory': True, 'override': 'abc'}
        get_url = self.url + 'env/qa/abc/'
        get.assert_called_once_with(get_url, params=params)
        json = self.get_history.json()
        self.assertEqual(history, json['History'])

    def test_validate_path(self):
        """
        You must specify something that looks like a path.
        It must start with '/', etc.
        """
        with self.assertRaises(InvalidCall):
            _validate_path('abc')
        with self.assertRaises(InvalidCall):
            _validate_path('/abc def')

    @patch('requests.Session.post')
    def test_set(self, post):
        post.return_value = build()
        self.settings.set('dev', '/abc', '', 1000)
        payload = {'value': 1000}
        params = {'override': ''}
        post_url = self.url + 'env/dev/abc/'
        post.assert_called_once_with(post_url, data=payload, params=params)

        call = lambda: self.settings.set('dev', '/abc', '', 1000)
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.post')
    def test_set_override(self, post):
        post.return_value = build()
        self.settings.set('dev', '/abc', self.name, 'some value')
        payload = {'value': 'some value'}
        post_url = self.url + 'env/dev/abc/'
        params = {'override': self.name}
        post.assert_called_once_with(post_url, data=payload, params=params)

    @patch('requests.Session.post')
    def test_set_protect(self, post):
        post.return_value = build()
        self.settings.set_protect('dev', '/abc', '', True)
        payload = {'protect': True}
        params = {'override': ''}
        post_url = self.url + 'env/dev/abc/'
        post.assert_called_once_with(post_url, data=payload, params=params)

        call = lambda: self.settings.set_protect('dev', '/abc', '', True)
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.post')
    def test_set_protect_override(self, post):
        post.return_value = build()
        self.settings.set_protect('dev', '/abc', self.name, True)
        payload = {'protect': True}
        post_url = self.url + 'env/dev/abc/'
        params = {'override': self.name}
        post.assert_called_once_with(post_url, data=payload, params=params)



# Copyright 2015 Digital Borderlands Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from cityhall import (
    Settings,
    _sanitize_url,
    _hash_password,
    _ensure_okay
)
from cityhall.errors import FailedCall, InvalidCall
from unittest import TestCase
from helper_funcs import (
    build,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
)
from mock import patch


class TestSettingsFuncs(TestCase):
    def test_sanitize_url(self):
        self.assertEqual('http://url/', _sanitize_url('http://url'))
        self.assertEqual('http://url/', _sanitize_url('http://url/'))

    def test_hash_password(self):
        self.assertEqual(
            '', _hash_password(''),
            "by convention, empty strings are passed as-is"
        )
        self.assertNotEqual('abc', _hash_password('abc'))
        self.assertNotEqual('', _hash_password('abc'))

    def test_ensure_okay(self):
        self.assertIsInstance(_ensure_okay(build()), dict)
        with self.assertRaises(FailedCall):
            _ensure_okay(build(status_code=400))
        with self.assertRaises(FailedCall):
            _ensure_okay(build(reply='Failure', message='abc'))


class TestAuthenticationAndDefaultEnvs(
    TestCase,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
):
    """
    This class the the authentication for the library.
    Authentication is done at the start, and maintained in the session.
    """
    def setUp(self):
        self.url = 'http://not.a.real.url/api/'
        self.username = 'test_user'

    @patch('requests.Session.get')
    @patch('requests.Session.post')
    def test_no_password_is_honored(self, post, get):
        """
        By convention, no password passes an empty string
        """
        post.return_value = build()
        get.return_value = build(update={'value': 'dev'})
        Settings(self.url, self.username, '')

        auth_url = self.url + 'auth/'
        post.assert_called_once_with(
            auth_url,
            data={'username': self.username, 'passhash': ''}
        )

    @patch('requests.Session.get')
    @patch('requests.Session.post')
    def test_password_is_hashed(self, post, get):
        """
        A password is passed in as plaintext and is hashed before the post
        """
        post.return_value = build()
        get.return_value = build(update={'value': 'dev'})
        Settings(self.url, self.username, 'abc')

        auth_url = self.url + 'auth/'
        post.assert_called_once_with(
            auth_url,
            data={'username': self.username, 'passhash': _hash_password('abc')}
        )

    @patch('requests.Session.get')
    @patch('requests.Session.post')
    def test_default_env_is_retrieved(self, post, get):
        """
        Upon logging in, the default environment is retrieved
        """
        post.return_value = build()
        get.return_value = build(update={'value': 'dev'})
        settings = Settings(self.url, self.username, 'abc')
        get.asssert_called_once_with(
            self.url + 'auth/user/' + self.username + '/default/'
        )
        self.assertEqual('dev', settings.default_env)

        self.failed_call_honored(
            settings.get_default_env, 'requests.Session.get'
        )
        self.logout_honored(settings.get_default_env, settings)

    @patch('requests.Session.delete')
    @patch('requests.Session.get')
    @patch('cityhall.requests.Session.post')
    def test_logging_out(self, post, get, delete):
        """
        Logging out hits the correct url
        """
        post.return_value = build()
        get.return_value = build(update={'value': 'dev'})
        delete.return_value = build()
        settings = Settings(self.url, self.username, '')
        settings.log_out()

        delete.assert_called_once_with(self.url + 'auth/')
        self.assertFalse(settings.logged_in)

    @patch('requests.Session.post')
    @patch('requests.Session.get')
    def test_setting_default_env(self, get, post):
        """
        A user should be able to set the environment, also.
        This test is here for completeness sake, since the call to get
        the default environment is part creating the Settings() class
        """
        get.return_value = build(update={'value': 'dev'})
        post.return_value = build(message='Updated default env')
        settings = Settings(self.url, self.username, '')
        settings.set_default_env('abc')

        post.assert_called_with(
            self.url + 'auth/user/' + self.username + '/default/',
            data={'env': 'abc'}
        )
        self.assertEqual('abc', settings.default_env)

        call = lambda: settings.set_default_env('abc')
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, settings)


class TestAuthEnvironments(
    TestCase,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
):
    def setUp(self):
        self.url = 'http://not.a.real.url/api/'
        self.name = 'test_user'
        self.password = ''
        with patch('requests.Session.post') as post:
            with patch('requests.Session.get') as get:
                post.return_value = build()
                get.return_value = build(update={'value': 'dev'})
                self.settings = Settings(self.url, self.name, self.password)

    @patch('requests.Session.get')
    def test_can_get_environment(self, get):
        """
        A user is able to get details for an environment
        """
        get.return_value = build(
            update={'Users': {'test_user': 4, 'user2': 1, 'guest': 1}}
        )
        env_users = self.settings.get_env('dev')

        self.assertEqual(3, len(env_users))
        self.assertIn('test_user', env_users)
        self.assertEqual(4, env_users['test_user'])

        call = lambda: self.settings.get_env('dev')
        self.failed_call_honored(call, 'requests.Session.get')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.post')
    def test_can_create_environment(self, post):
        """
        A user is able to create an environment
        """
        post.return_value = build()
        self.settings.create_env('dev2')
        post.assert_called_once_with(self.url + 'auth/env/dev2/')

        call = lambda: self.settings.create_env('dev')
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, self.settings)


class TestAuthUsers(
    TestCase,
    TestRaisesLoggedOutMixin,
    TestFailureIsReturnedMixin,
):
    def setUp(self):
        self.url = 'http://not.a.real.url/api/'
        self.name = 'test_user'
        self.password = ''
        with patch('requests.Session.post') as post:
            with patch('requests.Session.get') as get:
                post.return_value = build()
                get.return_value = build(update={'value': 'dev'})
                self.settings = Settings(self.url, self.name, self.password)

    @patch('requests.Session.get')
    def test_get_user(self, get):
        """
        Get information about a user, his permissions on all environments
        """
        get.return_value = build(update={
            'Environments': {'dev': 4, 'auto': 1, 'users': 1}
        })
        rights = self.settings.get_user('test_user')

        self.assertEqual(3, len(rights))
        self.assertIn('dev', rights)
        self.assertEqual(4, rights['dev'])

        get.asssert_called_once_with(self.url + 'auth/user/test_user/')
        call = lambda: self.settings.get_user('test_user')
        self.failed_call_honored(call, 'requests.Session.get')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.post')
    def test_create_user(self, post):
        """
        Create a user
        """
        post.return_value = build()
        self.settings.create_user('user2', '')
        auth_url = self.url + 'auth/user/user2/'
        post.assert_called_with(auth_url, data={'passhash': ''})

        self.settings.create_user('user2', 'abc')
        abc_hash = _hash_password('abc')
        post.assert_called_with(auth_url, data={'passhash': abc_hash})

        call = lambda: self.settings.create_user('user2', 'abc')
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.put')
    def test_update_password(self, put):
        """
        Update your own password
        """
        put.return_value = build()
        self.settings.update_password('')
        auth_url = self.url + 'auth/user/' + self.name + '/'
        put.assert_called_with(auth_url, data={'passhash': ''})

        self.settings.update_password('abc')
        abc_hash = _hash_password('abc')
        put.assert_called_with(auth_url, data={'passhash': abc_hash})

        call = lambda: self.settings.update_password('abc')
        self.failed_call_honored(call, 'requests.Session.put')
        self.logout_honored(call, self.settings)

    @patch('requests.Session.delete')
    def test_delete_user(self, delete):
        """
        Delete a user
        """
        delete.return_value = build()
        self.settings.delete_user('other_user')
        auth_url = self.url + 'auth/user/other_user/'
        delete.assert_called_once_with(auth_url)

        call = lambda: self.settings.delete_user('other_user')
        self.failed_call_honored(call, 'requests.Session.delete')
        self.logout_honored(call, self.settings)


class TestAuthGrant(
    TestCase,
    TestFailureIsReturnedMixin,
    TestRaisesLoggedOutMixin
):
    def setUp(self):
        self.url = 'http://not.a.real.url/api/'
        self.name = 'test_user'
        self.password = ''
        with patch('requests.Session.post') as post:
            with patch('requests.Session.get') as get:
                post.return_value = build()
                get.return_value = build(update={'value': 'dev'})
                self.settings = Settings(self.url, self.name, self.password)

    def test_grant_rights_validation(self):
        """
        Rights should be a value of 0, 1, 2, 3, or 4
        """
        with self.assertRaises(InvalidCall):
            self.settings.grant_rights('dev', 1, 0)
        with self.assertRaises(InvalidCall):
            self.settings.grant_rights(1, 'abc', 0)
        with self.assertRaises(InvalidCall):
            self.settings.grant_rights('dev', 'abc', 5)
        with self.assertRaises(InvalidCall):
            self.settings.grant_rights('dev', 'abc', -1)
        with self.assertRaises(InvalidCall):
            self.settings.grant_rights('dev', 'abc', 1.1)

    @patch('requests.Session.post')
    def test_grant_rights(self, post):
        post.return_value = build()
        self.settings.grant_rights('dev', 'abc', 2)

        post.assert_called_once_with(
            self.url + 'auth/grant/',
            data={
                'env': 'dev',
                'user': 'abc',
                'rights': 2
            }
        )
        call = lambda: self.settings.grant_rights('dev', 'abc', 2)
        self.failed_call_honored(call, 'requests.Session.post')
        self.logout_honored(call, self.settings)



# Copyright 2015 Digital Borderlands Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
from errors import NotLoggedIn, FailedCall, InvalidCall
import hashlib
from six import text_type


def _sanitize_url(url):
    return url if url[-1] == '/' else url + '/'


def _hash_password(password):
    """
    Return the City Hall hash for the a given password

    :param password: the plaintext password
    :return: md5 hashed password
    """
    if not password:
        return ''

    md5 = hashlib.md5()
    md5.update(password)
    return text_type(md5.hexdigest())


def _ensure_okay(resp):
    if resp.status_code != 200:
        raise FailedCall("Status code not 200: {}".format(resp.status_code))
    ret = resp.json()
    if ret['Response'] == 'Ok':
        return ret
    raise FailedCall(ret.get('Message', 'No message given for failure'))


def _validate_path(path):
    if path[0] != '/' or path.find(' ') > 0:
        raise InvalidCall("Given path is invalid")


class Settings(object):
    def __init__(self, url, username, password):
        self.session = requests.Session()
        self.url = _sanitize_url(url)
        self.name = username
        self.logged_in = False

        auth_url = self.url + 'auth/'
        passhash = _hash_password(password)
        payload = {'username': self.name, 'passhash': passhash}
        resp = self.session.post(auth_url, data=payload)
        _ensure_okay(resp)

        self.logged_in = True
        self.default_env = None
        self.get_default_env()

    def _ensure_logged_in(self):
        if not self.logged_in:
            raise NotLoggedIn()

    def get_default_env(self):
        """
        Returns the default environment for this user. If you use get()
        without specifying 'env', it will use this environment
        """
        self._ensure_logged_in()

        env_url = self.url + 'auth/user/{}/default/'.format(self.name)
        resp = self.session.get(env_url)
        env = _ensure_okay(resp)
        self.default_env = env['value']

    def set_default_env(self, env):
        """
        Sets the default environment.  This will be honored by the call to
        get() without specifying 'env'.

        Note that no checking is done to make sure that the environment
        specified exists or that the user has read permissions to it.
        Subsequent calls to get() might fail in that case.

        :param env: the environment to set as default.
        """
        self._ensure_logged_in()

        env_url = self.url + 'auth/user/{}/default/'.format(self.name)
        payload = {'env': env}
        resp = self.session.post(env_url, data=payload)
        _ensure_okay(resp)
        self.default_env = env

    def log_out(self):
        """
        Logs the user out. Future calls to the library will raise NotLoggedIn.
        This function is idempotent.
        """
        if self.logged_in:
            self.session.delete(self.url + 'auth/')
            self.logged_in = None

    def get_env(self, env):
        """
        Gets information for the given environment.  For example,
        users who have permissions to it, and their permission level.

        Note that Read permissions are required for this to work.

        :param env: The environment to look at.
        :return: List of users and their read permissions
        """
        self._ensure_logged_in()
        env_url = self.url + 'auth/env/' + env + '/'
        resp = self.session.get(env_url)
        json = _ensure_okay(resp)
        return json['Users']

    def create_env(self, env):
        """
        Allows the user to create an environment. Sets that user up
        as having Grant rights to that environment

        :param env: the environment to create
        """
        self._ensure_logged_in()
        env_url = self.url + 'auth/env/' + env + '/'
        resp = self.session.post(env_url)
        _ensure_okay(resp)

    def get_user(self, user):
        """
        Get user rights for a particular user.

        :param user: the user to query for.
        :return: dict, where keys are environments, and the values are
            rights for that user
        """
        self._ensure_logged_in()
        user_url = self.url + 'auth/user/' + user + '/'
        resp = self.session.get(user_url)
        json = _ensure_okay(resp)
        return json['Environments']

    def create_user(self, user, password):
        """
        Create a user with that password.

        :param user: The user to create
        :param password: The plaintext password
        """
        self._ensure_logged_in()
        user_url = self.url + 'auth/user/' + user + '/'
        payload = {'passhash': _hash_password(password)}
        resp = self.session.post(user_url, data=payload)
        _ensure_okay(resp)

    def update_password(self, password):
        """
        Update your own password.

        :param password: The plaintext password
        """
        self._ensure_logged_in()
        user_url = self.url + 'auth/user/' + self.name + '/'
        payload = {'passhash': _hash_password(password)}
        resp = self.session.put(user_url, data=payload)
        _ensure_okay(resp)

    def delete_user(self, user):
        """
        Delete a user. Deletion can only happen if a user's environments have
        Grant permissions for the current user, or the user has Write
        permissions to the User environment.  If deletion fails, this will
        raise FailedCall.

        :param user: The user to be deleted
        """
        self._ensure_logged_in()
        user_url = self.url + 'auth/user/' + user + '/'
        resp = self.session.delete(user_url)
        _ensure_okay(resp)

    def grant_rights(self, env, user, rights):
        """
        Grant user 'user' 'rights' on environment 'env'.  Note that in order
        to successfully do this, the current user must himself have Grant
        rights on 'env'.

        :param env: string - the environment to set the rights
        :param user: string - the user for whom to set the rights
        :param rights: int - the rights to set. Must be one of 0 - no rights,
            1 - Read, 2 - Read Protected, 3 - Write, 4 - Grant.
        """
        if isinstance(env, text_type) or isinstance(env, str):
            env = text_type(env)
        else:
            raise InvalidCall("Environment must be a string")
        if isinstance(user, text_type) or isinstance(user, str):
            user = text_type(user)
        else:
            raise InvalidCall("User must be a string")
        if not isinstance(rights, int):
            raise InvalidCall("Rights must be an integer 0-4")
        if rights < 0 or rights > 4:
            raise InvalidCall("Rights must be an integer 0-4")

        self._ensure_logged_in()
        grant_url = self.url + 'auth/grant/'
        payload = {'user': user, 'env': env, 'rights': rights}
        resp = self.session.post(grant_url, data=payload)
        _ensure_okay(resp)

    def _get_raw(self, env, path, params):
        _validate_path(path)
        self._ensure_logged_in()
        env = env or self.default_env
        get_url = _sanitize_url(self.url + 'env/' + env + path)
        resp = self.session.get(get_url, params=params)
        return _ensure_okay(resp)

    def get(self, path, env=None, override=None, view_raw=False):
        params = None if override is None else {'override': override}
        json = self._get_raw(env, path, params)
        return json if view_raw else json['value']

    def get_history(self, path, env=None, override=None):
        params = {} if override is None else {'override': override}
        params['viewhistory'] = True
        json = self._get_raw(env, path, params)
        return json['History']

    def get_children(self, path, env=None, override=None):
        params = {} if override is None else {'override': override}
        params['viewchildren'] = True
        json = self._get_raw(env, path, params)
        return json['children']

    def _set_raw(self, env, path, override, payload):
        _validate_path(path)
        self._ensure_logged_in()
        set_url = _sanitize_url(self.url + 'env/' + env + path)
        params = {'override': override}
        resp = self.session.post(set_url, data=payload, params=params)
        _ensure_okay(resp)

    def set(self, env, path, override, value):
        payload = {'value': value}
        self._set_raw(env, path, override, payload)

    def set_protect(self, env, path, override, protect):
        payload = {'protect': protect}
        self._set_raw(env, path, override, payload)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


setup(
    name = 'cityhall',
    packages = ['cityhall'], # this must be the same as the name above
    version = '0.0.10',
    description = 'A library for accessing City Hall Setting Server',
    license = 'AGPL',
    author = 'Alex Popa',
    author_email = 'alex.popa@digitalborderlands.com',
    url = 'https://github.com/f00f-nyc/cityhall-python',
    download_url = 'https://codeload.github.com/f00f-nyc/cityhall-python/legacy.tar.gz/v0.0.10',
    install_requires=['requests==2.7.0','six==1.9.0'],
    keywords = ['cityhall', 'enterprise settings', 'settings', 'settings server', 'cityhall', 'City Hall'],
    test_suite='test',
    tests_require=['requests==2.7.0','six==1.9.0','mock==1.0.1'],
    classifiers = [],
)